@@ -1,7 +1,7 @@
 <?php
 /*
 ** Zabbix
-** Copyright (C) 2001-2017 Zabbix SIA
+** Copyright (C) 2001-2018 Zabbix SIA
 **
 ** This program is free software; you can redistribute it and/or modify
 ** it under the terms of the GNU General Public License as published by
@@ -164,103 +164,13 @@ function get_hosts_by_graphid($graphid) {
  */
 function get_min_itemclock_by_graphid($graphid) {
 	$items = DBfetchArray(DBselect(
-		'SELECT DISTINCT i.itemid,i.value_type,i.history,i.trends'.
+		'SELECT DISTINCT i.itemid,i.value_type,i.history,i.trends,i.hostid'.
 		' FROM items i,graphs_items gi'.
 		' WHERE i.itemid=gi.itemid'.
 			' AND gi.graphid='.zbx_dbstr($graphid)
 	));
 
-	return get_min_itemclock_by_itemid($items);
-}
-
-/**
- * Return the time of the 1st appearance of items in history or trends.
- *
- * @param array $items
- * @param array $items[]['itemid']
- * @param array $items[]['value_type']
- * @param array $items[]['history']
- * @param array $items[]['trends']
- *
- * @return int (unixtime)
- */
-function get_min_itemclock_by_itemid($items) {
-	$item_types = [
-		ITEM_VALUE_TYPE_FLOAT => [],
-		ITEM_VALUE_TYPE_STR => [],
-		ITEM_VALUE_TYPE_LOG => [],
-		ITEM_VALUE_TYPE_UINT64 => [],
-		ITEM_VALUE_TYPE_TEXT => []
-	];
-
-	$max = ['history' => 0, 'trends' => 0];
-
-	foreach ($items as $item) {
-		$item_types[$item['value_type']][] = $item['itemid'];
-
-		if ($item['value_type'] == ITEM_VALUE_TYPE_FLOAT || $item['value_type'] == ITEM_VALUE_TYPE_UINT64) {
-			$max['history'] = max($max['history'], (int) $item['history']);
-			$max['trends'] = max($max['trends'], (int) $item['trends']);
-		}
-	}
-
-	$sql_unions = [];
-	foreach ($item_types as $type => $itemids) {
-		if ($itemids) {
-			switch ($type) {
-				case ITEM_VALUE_TYPE_FLOAT:
-					$sql_from = ($max['history'] > $max['trends']) ? 'history' : 'trends';
-					break;
-				case ITEM_VALUE_TYPE_STR:
-					$sql_from = 'history_str';
-					break;
-				case ITEM_VALUE_TYPE_LOG:
-					$sql_from = 'history_log';
-					break;
-				case ITEM_VALUE_TYPE_UINT64:
-					$sql_from = ($max['history'] > $max['trends']) ? 'history_uint' : 'trends_uint';
-					break;
-				case ITEM_VALUE_TYPE_TEXT:
-					$sql_from = 'history_text';
-					break;
-				default:
-					$sql_from = 'history';
-			}
-
-			foreach ($itemids as $itemid) {
-				$sql_unions[] =
-					'SELECT MIN(h.clock) AS clock'.
-					' FROM '.$sql_from.' h'.
-					' WHERE h.itemid='.zbx_dbstr($itemid);
-			}
-		}
-	}
-
-	$row = DBfetch(DBselect(
-		'SELECT MIN(h.clock) AS min_clock'.
-		' FROM ('.implode(' UNION ALL ', $sql_unions).') h'
-	));
-	$min_clock = $row['min_clock'];
-
-	// in case DB clock column is corrupted having negative numbers, return min clock from max possible history storage
-	if ($min_clock == 0) {
-		if ($item_types[ITEM_VALUE_TYPE_FLOAT] || $item_types[ITEM_VALUE_TYPE_UINT64]) {
-			$min_clock = time() - SEC_PER_DAY * max($max['history'], $max['trends']);
-
-			/*
-			 * In case history storage exceeds the maximum time difference between current year and minimum 1970
-			 * (for example year 2014 - 200 years < year 1970), correct year to 1970 (unix time timestamp 0).
-			 */
-			if ($min_clock < 0) {
-				$min_clock = 0;
-			}
-		}
-		else {
-			$min_clock = time() - SEC_PER_YEAR;
-		}
-	}
-
-	return $min_clock;
+	return Manager::History()->getMinClock($items);
 }
 
 function getGraphByGraphId($graphId) {
@@ -376,51 +286,6 @@ function copyGraphToHost($graphid, $hostid) {
 	return API::Graph()->create($graph);
 }
 
-function navigation_bar_calc($idx = null, $idx2 = 0, $update = false) {
-	if (!empty($idx)) {
-		if ($update) {
-			if (!empty($_REQUEST['period']) && $_REQUEST['period'] >= ZBX_MIN_PERIOD) {
-				CProfile::update($idx.'.period', $_REQUEST['period'], PROFILE_TYPE_INT, $idx2);
-			}
-			if (!empty($_REQUEST['stime'])) {
-				CProfile::update($idx.'.stime', $_REQUEST['stime'], PROFILE_TYPE_STR, $idx2);
-			}
-		}
-		$_REQUEST['period'] = getRequest('period', CProfile::get($idx.'.period', ZBX_PERIOD_DEFAULT, $idx2));
-		$_REQUEST['stime'] = getRequest('stime', CProfile::get($idx.'.stime', null, $idx2));
-	}
-
-	$_REQUEST['period'] = getRequest('period', ZBX_PERIOD_DEFAULT);
-	$_REQUEST['stime'] = getRequest('stime');
-
-	if ($_REQUEST['period'] < ZBX_MIN_PERIOD) {
-		show_error_message(_n('Minimum time period to display is %1$s minute.',
-			'Minimum time period to display is %1$s minutes.',
-			(int) ZBX_MIN_PERIOD / SEC_PER_MIN
-		));
-		$_REQUEST['period'] = ZBX_MIN_PERIOD;
-	}
-	elseif ($_REQUEST['period'] > ZBX_MAX_PERIOD) {
-		show_error_message(_n('Maximum time period to display is %1$s day.',
-			'Maximum time period to display is %1$s days.',
-			(int) ZBX_MAX_PERIOD / SEC_PER_DAY
-		));
-		$_REQUEST['period'] = ZBX_MAX_PERIOD;
-	}
-
-	if (!empty($_REQUEST['stime'])) {
-		$time = zbxDateToTime($_REQUEST['stime']);
-		if (($time + $_REQUEST['period']) > time()) {
-			$_REQUEST['stime'] = date(TIMESTAMP_FORMAT, time() - $_REQUEST['period']);
-		}
-	}
-	else {
-		$_REQUEST['stime'] = date(TIMESTAMP_FORMAT, time() - $_REQUEST['period']);
-	}
-
-	return $_REQUEST['period'];
-}
-
 function get_next_color($palettetype = 0) {
 	static $prev_color = ['dark' => true, 'color' => 0, 'grad' => 0];
 
